% PLOT_SURFACE_SPHERE
% 	Plots the surface of Bezier on the Euclidean space, given the
% 	structure pb.
% 
% Input: pb: the structure of the Bezier solution.
%
% Output: a magnificent plot with:
% 	- The surface of Bezier
% 	- The control points displayed in green;
% 	- The interpolation points displayed in red;
% 	- One patch on two is blue, the other is yellow.

function plot_surface_eucl(pb)

    % Variables
    t 		= pb.t;
	curve 	= pb.curve;
	p 		= pb.interp;
	b 		= pb.control;
	
	% Prepare the sphere to plot on it
	[x,y,z] = sphere(20);
	surf(x*0.99,y*0.99,z*0.99,'FaceColor',[255 215 0]/255,'FaceAlpha',0.5,'EdgeAlpha',0.5);
	
	hold on;

	% Lines following in x.
	tt = round(size(pb.curve,1)/(pb.n-1));
	for k = 1:(pb.n-1)*tt
		if (mod(floor((k-1)/tt),2)) == 0
			plot3(curve(k,:,1,1),curve(k,:,1,2),curve(k,:,1,3),'b', 'LineWidth',1.5);	
			hold on;
		else
			plot3(curve(k,:,1,1),curve(k,:,1,2),curve(k,:,1,3),'--k', 'LineWidth',1.5);	
			hold on;
		end
	end

	% Control points
	for i = 1:size(b,1)
	for j = 1:size(b,2)
		toplot = b{i,j};
		plot3(toplot(1),toplot(2),toplot(3),'.','Color',[0 0.7 0],'MarkerSize',30);
	end
	end

	% Interpolation points
	for i = 1:size(p,1)
	for j = 1:size(p,2)
		toplot = p{i,j};
		plot3(toplot(1),toplot(2),toplot(3),'.r','MarkerSize',30);
	end
	end

	
	axis off;
	axis equal;
end
